<?php

namespace Drupal\menu_migration\Plugin\menu_migration\ImportSource;

use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a codebase import destination.
 *
 * @MenuMigrationSource(
 *   id = "codebase",
 *   label = @Translation("Codebase"),
 *   multiple = true,
 *   cli = true
 * )
 */
class Codebase extends ImportSourceBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'import_path' => '../config/menu_migration/DIRECTORY',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['import_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Import directory path'),
      '#description' => $this->t("Enter the path from where the items should be imported from (relative to Drupal root - %root_path), no trailing slash. <br>Your menu(s) will be retrieved from this exact location. <br>E.g. If the defined path is %import_path, the selected menus will be imported from %import_location.", [
        '%root_path' => DRUPAL_ROOT,
        '%import_path' => $this->configuration['import_path'],
        '%import_location' => $this->getSourceDirectory(),
      ]),
      '#default_value' => $this->configuration['import_path'],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
    // Ensure that there are no trailing slashes, nor white spaces.
    $form_state->setValue('import_path', trim(rtrim($form_state->getValue('import_path'), '/')));
  }

  /**
   * {@inheritdoc}
   */
  public function importMenu(string $menuName) {
    $format = $this->getFormatPlugin();
    $directory = $this->getSourceDirectory();
    // @todo This next line throws warning if the file doesn't exist.
    if ($contents = file_get_contents("{$directory}/{$menuName}.json")) {
      $items = $format->decode($contents);
      $this->menuMigrationService->generateMenuItems($items, $menuName);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getImportDescription() {
    $menus = $this->getMenus();
    $path = $this->configuration['import_path'];
    $extension = $this->getFormatPlugin()->defaultExtension();
    $locations = array_map(function ($menu) use ($path, $extension) {
      $location = [
        $path,
        '/',
        $menu,
        '.',
        $extension,
      ];
      return implode('', $location);
    }, $menus);
    return $this->formatPlural(
      count($locations),
      'Your menu will be imported from %paths',
      'Your menus will be imported from: %paths',
      ['%paths' => implode(', ', $locations)],
    );
  }

  /**
   * Gets the source directory for the import source.
   *
   * @return string
   *   Returns the full path to the source directory.
   */
  protected function getSourceDirectory() {
    $path = [
      DRUPAL_ROOT,
      $this->configuration['import_path'],
    ];
    return implode('/', $path);
  }

}
